from typing import Generator
from uuid import UUID
from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.price.price_pb2 import PriceProto
from fintekkers.models.util.uuid_pb2 import UUIDProto
from fintekkers.requests.price.query_price_request_pb2 import QueryPriceRequestProto
from fintekkers.requests.price.query_price_response_pb2 import QueryPriceResponseProto

from fintekkers.wrappers.models.price import Price
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil
from fintekkers.wrappers.requests.price import CreatePriceRequest, QueryPriceRequest

from fintekkers.wrappers.services.util.Environment import EnvConfig, ServiceType

from fintekkers.services.price_service.price_service_pb2_grpc import PriceStub

class PriceService:
    def __init__(self):
        print("PriceService connecting to: " + EnvConfig.api_url(ServiceType.PRICE_SERVICE))
        self.stub = PriceStub(EnvConfig.get_channel(ServiceType.PRICE_SERVICE))

    def search(self, request: QueryPriceRequest) -> Generator[Price, None, None]:
        responses = self.stub.Search(request=request.proto)

        try:
            while not responses._is_complete():
                response: QueryPriceResponseProto = responses.next()

                for price_proto in response.price_response:
                    yield Price(price_proto)
        except StopIteration:
            pass
        except Exception as e:
            print(e)

        # This will send the cancel message to the server to kill the connection
        responses.cancel()

    def create_or_update(self, request: CreatePriceRequest):
        return self.stub.CreateOrUpdate(request.proto)

    def get_price_by_uuid(self,uuid: UUID) -> Price:
        """
        Parameters:
            A UUID

        Returns:
            request (Price): Returns the Price proto for the UUID, or None if doesn't exist
        """
        uuid_proto = UUIDProto(raw_uuid=uuid.bytes)

        # request: QueryPriceRequest = QueryPriceRequest.create_query_request(
        #     {
        #         FieldProto.ID: uuid_proto,
        #     },
        #     frequency=None,
        #     start_date=None,
        #     end_date=None
        # )

        request:QueryPriceRequestProto = QueryPriceRequestProto(
            uuIds=[uuid_proto]
        )

        prices = self.stub.GetByIds(request).price_response

        for price in prices:
            return Price(price)
        
    def list_ids(self) -> list[UUID]:
        request: QueryPriceRequest = QueryPriceRequest.create_query_request(
            fields={},
            frequency=None,
            start_date=None,
            end_date=None
        )

        response: QueryPriceResponseProto = self.stub.ListIds(request.proto)

        ids: list[UUID] = []

        for price_proto in response.price_response:
            price_proto: PriceProto
            price_id = price_proto.uuid
            uuid: UUID = ProtoSerializationUtil.deserialize(price_id).as_uuid()
            ids.append(uuid)

        return ids



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/price/query_price_request.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/price/query_price_request.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.util import uuid_pb2 as fintekkers_dot_models_dot_util_dot_uuid__pb2
from fintekkers.models.util import local_timestamp_pb2 as fintekkers_dot_models_dot_util_dot_local__timestamp__pb2
from fintekkers.models.position import position_filter_pb2 as fintekkers_dot_models_dot_position_dot_position__filter__pb2
from fintekkers.models.util import date_range_pb2 as fintekkers_dot_models_dot_util_dot_date__range__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3fintekkers/requests/price/query_price_request.proto\x12\x19\x66intekkers.requests.price\x1a!fintekkers/models/util/uuid.proto\x1a,fintekkers/models/util/local_timestamp.proto\x1a\x30\x66intekkers/models/position/position_filter.proto\x1a\'fintekkers/models/util/date_range.proto\"\xca\x03\n\x16QueryPriceRequestProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x30\n\x05uuIds\x18\x15 \x03(\x0b\x32!.fintekkers.models.util.UUIDProto\x12K\n\x12search_price_input\x18\x16 \x01(\x0b\x32/.fintekkers.models.position.PositionFilterProto\x12:\n\x05\x61s_of\x18\x17 \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12\x41\n\tfrequency\x18\x18 \x01(\x0e\x32..fintekkers.requests.price.PriceFrequencyProto\x12?\n\x07horizon\x18\x19 \x01(\x0e\x32,.fintekkers.requests.price.PriceHorizonProtoH\x00\x12<\n\ndate_range\x18\x1a \x01(\x0b\x32&.fintekkers.models.util.DateRangeProtoH\x00\x42\x0c\n\ntime_range*\xc5\x01\n\x13PriceFrequencyProto\x12\x1f\n\x1bPRICE_FREQUENCY_UNSPECIFIED\x10\x00\x12\x1a\n\x16PRICE_FREQUENCY_WEEKLY\x10\n\x12\x19\n\x15PRICE_FREQUENCY_DAILY\x10\x14\x12\x1a\n\x16PRICE_FREQUENCY_HOURLY\x10\x1e\x12\x1a\n\x16PRICE_FREQUENCY_MINUTE\x10(\x12\x1e\n\x1aPRICE_FREQUENCY_EVERY_TICK\x10Z*\xa1\x02\n\x11PriceHorizonProto\x12\x1d\n\x19PRICE_HORIZON_UNSPECIFIED\x10\x00\x12\x17\n\x13PRICE_HORIZON_1_DAY\x10\x01\x12\x18\n\x14PRICE_HORIZON_5_DAYS\x10\x02\x12\x18\n\x14PRICE_HORIZON_1_WEEK\x10\x03\x12\x19\n\x15PRICE_HORIZON_1_MONTH\x10\x04\x12\x1a\n\x16PRICE_HORIZON_6_MONTHS\x10\x05\x12\x18\n\x14PRICE_HORIZON_1_YEAR\x10\x06\x12\x18\n\x14PRICE_HORIZON_5_YEAR\x10\x07\x12\x15\n\x11PRICE_HORIZON_MAX\x10\x08\x12\x1e\n\x1aPRICE_HORIZON_YEAR_TO_DATE\x10\tB\x1b\x42\x17QueryPriceRequestProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.price.query_price_request_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\027QueryPriceRequestProtosP\001'
  _globals['_PRICEFREQUENCYPROTO']._serialized_start=716
  _globals['_PRICEFREQUENCYPROTO']._serialized_end=913
  _globals['_PRICEHORIZONPROTO']._serialized_start=916
  _globals['_PRICEHORIZONPROTO']._serialized_end=1205
  _globals['_QUERYPRICEREQUESTPROTO']._serialized_start=255
  _globals['_QUERYPRICEREQUESTPROTO']._serialized_end=713
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/services/price-service/price_service.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/services/price-service/price_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.requests.price import query_price_request_pb2 as fintekkers_dot_requests_dot_price_dot_query__price__request__pb2
from fintekkers.requests.price import query_price_response_pb2 as fintekkers_dot_requests_dot_price_dot_query__price__response__pb2
from fintekkers.requests.price import create_price_request_pb2 as fintekkers_dot_requests_dot_price_dot_create__price__request__pb2
from fintekkers.requests.price import create_price_response_pb2 as fintekkers_dot_requests_dot_price_dot_create__price__response__pb2
from fintekkers.requests.util.errors import summary_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_summary__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5fintekkers/services/price-service/price_service.proto\x12!fintekkers.services.price_service\x1a\x33\x66intekkers/requests/price/query_price_request.proto\x1a\x34\x66intekkers/requests/price/query_price_response.proto\x1a\x34\x66intekkers/requests/price/create_price_request.proto\x1a\x35\x66intekkers/requests/price/create_price_response.proto\x1a-fintekkers/requests/util/errors/summary.proto2\xd1\x05\n\x05Price\x12y\n\x0e\x43reateOrUpdate\x12\x32.fintekkers.requests.price.CreatePriceRequestProto\x1a\x33.fintekkers.requests.price.CreatePriceResponseProto\x12q\n\x08GetByIds\x12\x31.fintekkers.requests.price.QueryPriceRequestProto\x1a\x32.fintekkers.requests.price.QueryPriceResponseProto\x12q\n\x06Search\x12\x31.fintekkers.requests.price.QueryPriceRequestProto\x1a\x32.fintekkers.requests.price.QueryPriceResponseProto0\x01\x12p\n\x07ListIds\x12\x31.fintekkers.requests.price.QueryPriceRequestProto\x1a\x32.fintekkers.requests.price.QueryPriceResponseProto\x12{\n\x16ValidateCreateOrUpdate\x12\x32.fintekkers.requests.price.CreatePriceRequestProto\x1a-.fintekkers.requests.util.errors.SummaryProto\x12x\n\x14ValidateQueryRequest\x12\x31.fintekkers.requests.price.QueryPriceRequestProto\x1a-.fintekkers.requests.util.errors.SummaryProtoB\x06\x88\x01\x01\x90\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.services.price_service.price_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\210\001\001\220\001\001'
  _globals['_PRICE']._serialized_start=356
  _globals['_PRICE']._serialized_end=1077
_builder.BuildServices(DESCRIPTOR, 'fintekkers.services.price_service.price_service_pb2', _globals)
# @@protoc_insertion_point(module_scope)



from google.protobuf import any_pb2 as _any_pb2
from fintekkers.models.position import field_pb2 as _field_pb2
from fintekkers.models.position import measure_pb2 as _measure_pb2
from fintekkers.models.util import decimal_value_pb2 as _decimal_value_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PositionFilterOperator(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_OPERATOR: _ClassVar[PositionFilterOperator]
    EQUALS: _ClassVar[PositionFilterOperator]
    NOT_EQUALS: _ClassVar[PositionFilterOperator]
    LESS_THAN: _ClassVar[PositionFilterOperator]
    LESS_THAN_OR_EQUALS: _ClassVar[PositionFilterOperator]
    MORE_THAN: _ClassVar[PositionFilterOperator]
    MORE_THAN_OR_EQUALS: _ClassVar[PositionFilterOperator]
UNKNOWN_OPERATOR: PositionFilterOperator
EQUALS: PositionFilterOperator
NOT_EQUALS: PositionFilterOperator
LESS_THAN: PositionFilterOperator
LESS_THAN_OR_EQUALS: PositionFilterOperator
MORE_THAN: PositionFilterOperator
MORE_THAN_OR_EQUALS: PositionFilterOperator

class MeasureMapEntry(_message.Message):
    __slots__ = ("measure", "measure_decimal_value")
    MEASURE_FIELD_NUMBER: _ClassVar[int]
    MEASURE_DECIMAL_VALUE_FIELD_NUMBER: _ClassVar[int]
    measure: _measure_pb2.MeasureProto
    measure_decimal_value: _decimal_value_pb2.DecimalValueProto
    def __init__(self, measure: _Optional[_Union[_measure_pb2.MeasureProto, str]] = ..., measure_decimal_value: _Optional[_Union[_decimal_value_pb2.DecimalValueProto, _Mapping]] = ...) -> None: ...

class FieldMapEntry(_message.Message):
    __slots__ = ("field", "field_value_packed", "enum_value", "string_value", "operator")
    FIELD_FIELD_NUMBER: _ClassVar[int]
    FIELD_VALUE_PACKED_FIELD_NUMBER: _ClassVar[int]
    ENUM_VALUE_FIELD_NUMBER: _ClassVar[int]
    STRING_VALUE_FIELD_NUMBER: _ClassVar[int]
    OPERATOR_FIELD_NUMBER: _ClassVar[int]
    field: _field_pb2.FieldProto
    field_value_packed: _any_pb2.Any
    enum_value: int
    string_value: str
    operator: PositionFilterOperator
    def __init__(self, field: _Optional[_Union[_field_pb2.FieldProto, str]] = ..., field_value_packed: _Optional[_Union[_any_pb2.Any, _Mapping]] = ..., enum_value: _Optional[int] = ..., string_value: _Optional[str] = ..., operator: _Optional[_Union[PositionFilterOperator, str]] = ...) -> None: ...



from fintekkers.models.util.lock import node_partition_pb2 as _node_partition_pb2
from fintekkers.models.util import endpoint_pb2 as _endpoint_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class LockRequestProto(_message.Message):
    __slots__ = ("object_class", "version", "node_partition", "endpoint")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    NODE_PARTITION_FIELD_NUMBER: _ClassVar[int]
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    node_partition: _node_partition_pb2.NodePartition
    endpoint: _endpoint_pb2.Endpoint
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., node_partition: _Optional[_Union[_node_partition_pb2.NodePartition, _Mapping]] = ..., endpoint: _Optional[_Union[_endpoint_pb2.Endpoint, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/util/lock/node_state.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/util/lock/node_state.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.util import local_timestamp_pb2 as fintekkers_dot_models_dot_util_dot_local__timestamp__pb2
from fintekkers.models.util import endpoint_pb2 as fintekkers_dot_models_dot_util_dot_endpoint__pb2
from fintekkers.models.util.lock import node_partition_pb2 as fintekkers_dot_models_dot_util_dot_lock_dot_node__partition__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,fintekkers/models/util/lock/node_state.proto\x12\x1b\x66intekkers.models.util.lock\x1a,fintekkers/models/util/local_timestamp.proto\x1a%fintekkers/models/util/endpoint.proto\x1a\x30\x66intekkers/models/util/lock/node_partition.proto\"\xfa\x01\n\tNodeState\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12=\n\tpartition\x18\x05 \x01(\x0b\x32*.fintekkers.models.util.lock.NodePartition\x12\x33\n\tend_point\x18\x06 \x01(\x0b\x32 .fintekkers.models.util.Endpoint\x12>\n\tlast_seen\x18\x07 \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12\x12\n\nis_expired\x18\x08 \x01(\x08\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.util.lock.node_state_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_NODESTATE']._serialized_start=213
  _globals['_NODESTATE']._serialized_end=463
# @@protoc_insertion_point(module_scope)



from fintekkers.requests.security import query_security_request_pb2 as _query_security_request_pb2
from fintekkers.requests.security import query_security_response_pb2 as _query_security_response_pb2
from fintekkers.requests.security import create_security_request_pb2 as _create_security_request_pb2
from fintekkers.requests.security import create_security_response_pb2 as _create_security_response_pb2
from fintekkers.requests.security import get_fields_response_pb2 as _get_fields_response_pb2
from fintekkers.requests.security import get_field_values_request_pb2 as _get_field_values_request_pb2
from fintekkers.requests.security import get_field_values_response_pb2 as _get_field_values_response_pb2
from fintekkers.requests.util.errors import summary_pb2 as _summary_pb2
from google.protobuf import empty_pb2 as _empty_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import service as _service
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class Security(_service.service): ...

class Security_Stub(Security): ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/transaction/query_transaction_response.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/transaction/query_transaction_response.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.transaction import transaction_pb2 as fintekkers_dot_models_dot_transaction_dot_transaction__pb2
from fintekkers.requests.transaction import query_transaction_request_pb2 as fintekkers_dot_requests_dot_transaction_dot_query__transaction__request__pb2
from fintekkers.requests.util.errors import summary_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_summary__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n@fintekkers/requests/transaction/query_transaction_response.proto\x12\x1f\x66intekkers.requests.transaction\x1a/fintekkers/models/transaction/transaction.proto\x1a?fintekkers/requests/transaction/query_transaction_request.proto\x1a-fintekkers/requests/util/errors/summary.proto\"\xc3\x02\n\x1dQueryTransactionResponseProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x61\n\x1a\x63reate_transaction_request\x18\x14 \x01(\x0b\x32=.fintekkers.requests.transaction.QueryTransactionRequestProto\x12M\n\x14transaction_response\x18\x1e \x03(\x0b\x32/.fintekkers.models.transaction.TransactionProto\x12I\n\x12\x65rrors_or_warnings\x18( \x01(\x0b\x32-.fintekkers.requests.util.errors.SummaryProtoB\"B\x1eQueryTransactionResponseProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.transaction.query_transaction_response_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\036QueryTransactionResponseProtosP\001'
  _globals['_QUERYTRANSACTIONRESPONSEPROTO']._serialized_start=263
  _globals['_QUERYTRANSACTIONRESPONSEPROTO']._serialized_end=586
# @@protoc_insertion_point(module_scope)



from fintekkers.models.util import local_timestamp_pb2 as _local_timestamp_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DateRangeProto(_message.Message):
    __slots__ = ("object_class", "version", "start", "end")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    START_FIELD_NUMBER: _ClassVar[int]
    END_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    start: _local_timestamp_pb2.LocalTimestampProto
    end: _local_timestamp_pb2.LocalTimestampProto
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., start: _Optional[_Union[_local_timestamp_pb2.LocalTimestampProto, _Mapping]] = ..., end: _Optional[_Union[_local_timestamp_pb2.LocalTimestampProto, _Mapping]] = ...) -> None: ...



from fintekkers.models.transaction import transaction_pb2 as _transaction_pb2
from fintekkers.requests.transaction import query_transaction_request_pb2 as _query_transaction_request_pb2
from fintekkers.requests.util.errors import summary_pb2 as _summary_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QueryTransactionResponseProto(_message.Message):
    __slots__ = ("object_class", "version", "create_transaction_request", "transaction_response", "errors_or_warnings")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    CREATE_TRANSACTION_REQUEST_FIELD_NUMBER: _ClassVar[int]
    TRANSACTION_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    ERRORS_OR_WARNINGS_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    create_transaction_request: _query_transaction_request_pb2.QueryTransactionRequestProto
    transaction_response: _containers.RepeatedCompositeFieldContainer[_transaction_pb2.TransactionProto]
    errors_or_warnings: _summary_pb2.SummaryProto
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., create_transaction_request: _Optional[_Union[_query_transaction_request_pb2.QueryTransactionRequestProto, _Mapping]] = ..., transaction_response: _Optional[_Iterable[_Union[_transaction_pb2.TransactionProto, _Mapping]]] = ..., errors_or_warnings: _Optional[_Union[_summary_pb2.SummaryProto, _Mapping]] = ...) -> None: ...



from datetime import timedelta
from dateutil.relativedelta import relativedelta

from fintekkers.models.security.tenor_type_pb2 import TenorTypeProto
from fintekkers.models.security.tenor_pb2 import TenorProto
class Tenor:
    UNKNOWN_TENOR = None
    type:TenorTypeProto = None
    tenor:relativedelta = None
    
    def __init__(self, type:TenorTypeProto, term:str=None):
        self.type = type
        if term != None:
            self.tenor = Tenor.from_tenor_description(term)
    
    
    def __str__(self) -> str:
        result = []
        type_name = TenorTypeProto.Name(self.get_type())
        result.append(type_name)

        if self.get_type() == TenorTypeProto.TERM:
            result.append(": ")
            result.append(self.get_tenor_description())

        return "".join(result)

    @classmethod
    def from_tenor_description(cls, tenor_description) -> relativedelta:
        if not tenor_description and tenor_description != "":
            return None
        
        return Tenor.parse_period(tenor_description)
    
    def get_type(self) -> TenorTypeProto:
        return self.type


    def get_type_name(self) -> str:
        return TenorTypeProto.Name(self.type)
    
    def get_tenor(self) -> relativedelta:
        return self.tenor
    
    def get_tenor_description(self) -> str:
        if self.get_type() == TenorTypeProto.UNKNOWN_TENOR_TYPE:
            return TenorTypeProto.Name(self.get_type())
        if self.get_type() == TenorTypeProto.PERPETUAL:
            return TenorTypeProto.Name(self.get_type())
        
        return Tenor.period_to_string(self.tenor)
    
    def as_proto(self) -> TenorProto:
        return TenorProto(
            tenor_type=self.type,
            term_value=self.get_tenor_description()
        )

    @staticmethod
    def period_to_string(period:relativedelta) -> str:
        years = period.years
        months = period.months
        weeks = period.days // 7
        days = period.days % 7
        
        result = ""
        if years > 0:
            result += f"{years}Y"
        if months > 0:
            result += f"{months}M"
        if weeks > 0:
            result += f"{weeks}W"
        if days > 0:
            result += f"{days}D"
        
        return result.strip()
    
    @staticmethod
    def parse_period(period_string) -> relativedelta:
        years = 0
        months = 0
        weeks = 0
        days = 0
        
        number_string = ""
        for c in period_string:
            if c.isdigit():
                number_string += c
            else:
                number = int(number_string)
                if c == 'Y':
                    years = number
                elif c == 'M':
                    if period_string.index(c) < len(period_string) - 1 and period_string[period_string.index(c) + 1] == 'W':
                        weeks = number
                        number_string = ""
                        continue
                    else:
                        months = number
                elif c == 'W':
                    weeks = number
                elif c == 'D':
                    days = number
                else:
                    raise ValueError("Invalid character in period string: {}".format(c))
                number_string = ""
        
        return relativedelta(days=days, weeks=weeks, months=months, years=years)




from fintekkers.requests.portfolio import create_portfolio_request_pb2 as _create_portfolio_request_pb2
from fintekkers.requests.portfolio import create_portfolio_response_pb2 as _create_portfolio_response_pb2
from fintekkers.requests.portfolio import query_portfolio_request_pb2 as _query_portfolio_request_pb2
from fintekkers.requests.portfolio import query_portfolio_response_pb2 as _query_portfolio_response_pb2
from fintekkers.requests.util.errors import summary_pb2 as _summary_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import service as _service
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class Portfolio(_service.service): ...

class Portfolio_Stub(Portfolio): ...



from fintekkers.models.util.lock import node_state_pb2 as _node_state_pb2
from fintekkers.requests.util.lock import lock_request_pb2 as _lock_request_pb2
from fintekkers.requests.util.errors import summary_pb2 as _summary_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class LockResponseProto(_message.Message):
    __slots__ = ("object_class", "version", "get_lock_request", "lock_response", "errors_or_warnings")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    GET_LOCK_REQUEST_FIELD_NUMBER: _ClassVar[int]
    LOCK_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    ERRORS_OR_WARNINGS_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    get_lock_request: _lock_request_pb2.LockRequestProto
    lock_response: _node_state_pb2.NodeState
    errors_or_warnings: _summary_pb2.SummaryProto
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., get_lock_request: _Optional[_Union[_lock_request_pb2.LockRequestProto, _Mapping]] = ..., lock_response: _Optional[_Union[_node_state_pb2.NodeState, _Mapping]] = ..., errors_or_warnings: _Optional[_Union[_summary_pb2.SummaryProto, _Mapping]] = ...) -> None: ...



from fintekkers.models.position import position_util_pb2 as _position_util_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PositionViewProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_POSITION_VIEW: _ClassVar[PositionViewProto]
    DEFAULT_VIEW: _ClassVar[PositionViewProto]
    STRATEGY_VIEW: _ClassVar[PositionViewProto]

class PositionTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_POSITION_TYPE: _ClassVar[PositionTypeProto]
    TRANSACTION: _ClassVar[PositionTypeProto]
    TAX_LOT: _ClassVar[PositionTypeProto]
UNKNOWN_POSITION_VIEW: PositionViewProto
DEFAULT_VIEW: PositionViewProto
STRATEGY_VIEW: PositionViewProto
UNKNOWN_POSITION_TYPE: PositionTypeProto
TRANSACTION: PositionTypeProto
TAX_LOT: PositionTypeProto

class PositionProto(_message.Message):
    __slots__ = ("object_class", "version", "position_view", "position_type", "measures", "fields")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    POSITION_VIEW_FIELD_NUMBER: _ClassVar[int]
    POSITION_TYPE_FIELD_NUMBER: _ClassVar[int]
    MEASURES_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    position_view: PositionViewProto
    position_type: PositionTypeProto
    measures: _containers.RepeatedCompositeFieldContainer[_position_util_pb2.MeasureMapEntry]
    fields: _containers.RepeatedCompositeFieldContainer[_position_util_pb2.FieldMapEntry]
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., position_view: _Optional[_Union[PositionViewProto, str]] = ..., position_type: _Optional[_Union[PositionTypeProto, str]] = ..., measures: _Optional[_Iterable[_Union[_position_util_pb2.MeasureMapEntry, _Mapping]]] = ..., fields: _Optional[_Iterable[_Union[_position_util_pb2.FieldMapEntry, _Mapping]]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/security/tenor.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/security/tenor.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.security import tenor_type_pb2 as fintekkers_dot_models_dot_security_dot_tenor__type__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&fintekkers/models/security/tenor.proto\x12\x1a\x66intekkers.models.security\x1a+fintekkers/models/security/tenor_type.proto\"\x87\x01\n\nTenorProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x12\n\nterm_value\x18\x05 \x01(\t\x12>\n\ntenor_type\x18\x06 \x01(\x0e\x32*.fintekkers.models.security.TenorTypeProtoB\x0f\x42\x0bTenorProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.security.tenor_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\013TenorProtosP\001'
  _globals['_TENORPROTO']._serialized_start=116
  _globals['_TENORPROTO']._serialized_end=251
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/util/lock/lock_response.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/util/lock/lock_response.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.util.lock import node_state_pb2 as fintekkers_dot_models_dot_util_dot_lock_dot_node__state__pb2
from fintekkers.requests.util.lock import lock_request_pb2 as fintekkers_dot_requests_dot_util_dot_lock_dot_lock__request__pb2
from fintekkers.requests.util.errors import summary_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_summary__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1fintekkers/requests/util/lock/lock_response.proto\x12\x1d\x66intekkers.requests.util.lock\x1a,fintekkers/models/util/lock/node_state.proto\x1a\x30\x66intekkers/requests/util/lock/lock_request.proto\x1a-fintekkers/requests/util/errors/summary.proto\"\x8f\x02\n\x11LockResponseProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12I\n\x10get_lock_request\x18\x03 \x01(\x0b\x32/.fintekkers.requests.util.lock.LockRequestProto\x12=\n\rlock_response\x18\x04 \x01(\x0b\x32&.fintekkers.models.util.lock.NodeState\x12I\n\x12\x65rrors_or_warnings\x18( \x01(\x0b\x32-.fintekkers.requests.util.errors.SummaryProtoB\x16\x42\x12LockResponseProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.util.lock.lock_response_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\022LockResponseProtosP\001'
  _globals['_LOCKRESPONSEPROTO']._serialized_start=228
  _globals['_LOCKRESPONSEPROTO']._serialized_end=499
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/position/position.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/position/position.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.position import position_util_pb2 as fintekkers_dot_models_dot_position_dot_position__util__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)fintekkers/models/position/position.proto\x12\x1a\x66intekkers.models.position\x1a.fintekkers/models/position/position_util.proto\"\xbc\x02\n\rPositionProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x44\n\rposition_view\x18\n \x01(\x0e\x32-.fintekkers.models.position.PositionViewProto\x12\x44\n\rposition_type\x18\x0b \x01(\x0e\x32-.fintekkers.models.position.PositionTypeProto\x12=\n\x08measures\x18\x14 \x03(\x0b\x32+.fintekkers.models.position.MeasureMapEntry\x12\x39\n\x06\x66ields\x18\x15 \x03(\x0b\x32).fintekkers.models.position.FieldMapEntry*S\n\x11PositionViewProto\x12\x19\n\x15UNKNOWN_POSITION_VIEW\x10\x00\x12\x10\n\x0c\x44\x45\x46\x41ULT_VIEW\x10\x01\x12\x11\n\rSTRATEGY_VIEW\x10\x02*L\n\x11PositionTypeProto\x12\x19\n\x15UNKNOWN_POSITION_TYPE\x10\x00\x12\x0f\n\x0bTRANSACTION\x10\x01\x12\x0b\n\x07TAX_LOT\x10\x02\x42\x12\x42\x0ePositionProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.position.position_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\016PositionProtosP\001'
  _globals['_POSITIONVIEWPROTO']._serialized_start=440
  _globals['_POSITIONVIEWPROTO']._serialized_end=523
  _globals['_POSITIONTYPEPROTO']._serialized_start=525
  _globals['_POSITIONTYPEPROTO']._serialized_end=601
  _globals['_POSITIONPROTO']._serialized_start=122
  _globals['_POSITIONPROTO']._serialized_end=438
# @@protoc_insertion_point(module_scope)



from datetime import datetime
from uuid import uuid4, UUID

from google.protobuf.timestamp_pb2 import Timestamp
from fintekkers.models.price.price_pb2 import PriceProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.models.util.uuid_pb2 import UUIDProto
from fintekkers.wrappers.models.security.security import Security
from fintekkers.wrappers.models.util.fintekkers_uuid import FintekkersUuid
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil


class Price:
    def __init__(self, proto: PriceProto):
        self.proto: PriceProto = proto

    def __str__(self):
        return f"ID[{self.get_uuid()}], Price[{self.get_price()}]"

    def get_price(self) -> float:
        price:float = ProtoSerializationUtil.deserialize(self.proto.price)
        return price

    def get_as_of(self) -> datetime:
        as_of: LocalTimestampProto = ProtoSerializationUtil.deserialize(self.proto.as_of)
        return as_of

    def get_uuid(self) -> UUID:
        uuid: FintekkersUuid = ProtoSerializationUtil.deserialize(self.proto.uuid)
        return uuid.as_uuid()


    @staticmethod
    def create_price(security:Security, price: float, as_of_date:Timestamp):
        uuid_value = uuid4()

        price = PriceProto(
            as_of=LocalTimestampProto(
                timestamp=as_of_date, time_zone="America/New_York"
            ),
            is_link=False,
            object_class="Portfolio",
            uuid=UUIDProto(raw_uuid=uuid_value.bytes),
            price=ProtoSerializationUtil.serialize(price),
            security=security.proto,
            version="0.0.1",
        )

        return Price(price)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/util/errors/error.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/util/errors/error.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.requests.util.errors import message_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_message__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+fintekkers/requests/util/errors/error.proto\x12\x1f\x66intekkers.requests.util.errors\x1a-fintekkers/requests/util/errors/message.proto\"\x80\x01\n\nErrorProto\x12\x38\n\x04\x63ode\x18\x01 \x01(\x0e\x32*.fintekkers.requests.util.errors.ErrorCode\x12\x38\n\x06\x64\x65tail\x18\x02 \x01(\x0b\x32(.fintekkers.requests.util.errors.Message\"\x82\x01\n\x0cWarningProto\x12\x38\n\x04\x63ode\x18\x01 \x01(\x0e\x32*.fintekkers.requests.util.errors.ErrorCode\x12\x38\n\x06\x64\x65tail\x18\x02 \x01(\x0b\x32(.fintekkers.requests.util.errors.Message*+\n\tErrorCode\x12\x11\n\rUNKNOWN_ERROR\x10\x00\x12\x0b\n\x07WARNING\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.util.errors.error_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_ERRORCODE']._serialized_start=391
  _globals['_ERRORCODE']._serialized_end=434
  _globals['_ERRORPROTO']._serialized_start=128
  _globals['_ERRORPROTO']._serialized_end=256
  _globals['_WARNINGPROTO']._serialized_start=259
  _globals['_WARNINGPROTO']._serialized_end=389
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/util/errors/summary.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/util/errors/summary.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.requests.util.errors import error_pb2 as fintekkers_dot_requests_dot_util_dot_errors_dot_error__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-fintekkers/requests/util/errors/summary.proto\x12\x1f\x66intekkers.requests.util.errors\x1a+fintekkers/requests/util/errors/error.proto\"\x8c\x01\n\x0cSummaryProto\x12;\n\x06\x65rrors\x18\x01 \x03(\x0b\x32+.fintekkers.requests.util.errors.ErrorProto\x12?\n\x08warnings\x18\x02 \x03(\x0b\x32-.fintekkers.requests.util.errors.WarningProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.util.errors.summary_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SUMMARYPROTO']._serialized_start=128
  _globals['_SUMMARYPROTO']._serialized_end=268
# @@protoc_insertion_point(module_scope)



from fintekkers.models.position import field_pb2 as _field_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GetFieldValuesRequestProto(_message.Message):
    __slots__ = ("object_class", "version", "field")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    FIELD_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    field: _field_pb2.FieldProto
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., field: _Optional[_Union[_field_pb2.FieldProto, str]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/security/bond/issuance.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/security/bond/issuance.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.security.bond import auction_type_pb2 as fintekkers_dot_models_dot_security_dot_bond_dot_auction__type__pb2
from fintekkers.models.util import decimal_value_pb2 as fintekkers_dot_models_dot_util_dot_decimal__value__pb2
from fintekkers.models.util import local_date_pb2 as fintekkers_dot_models_dot_util_dot_local__date__pb2
from fintekkers.models.util import local_timestamp_pb2 as fintekkers_dot_models_dot_util_dot_local__timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.fintekkers/models/security/bond/issuance.proto\x12\x1f\x66intekkers.models.security.bond\x1a\x32\x66intekkers/models/security/bond/auction_type.proto\x1a*fintekkers/models/util/decimal_value.proto\x1a\'fintekkers/models/util/local_date.proto\x1a,fintekkers/models/util/local_timestamp.proto\"\xcd\x06\n\rIssuanceProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12:\n\x05\x61s_of\x18\x06 \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12?\n\nvalid_from\x18\x08 \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12=\n\x08valid_to\x18\t \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12I\n\x19\x61uction_announcement_date\x18\x14 \x01(\x0b\x32&.fintekkers.models.util.LocalDateProto\x12\x42\n\x12\x61uction_issue_date\x18\x15 \x01(\x0b\x32&.fintekkers.models.util.LocalDateProto\x12T\n!post_auction_outstanding_quantity\x18\x16 \x01(\x0b\x32).fintekkers.models.util.DecimalValueProto\x12J\n\x17\x61uction_offering_amount\x18\x17 \x01(\x0b\x32).fintekkers.models.util.DecimalValueProto\x12G\n\x0c\x61uction_type\x18\x18 \x01(\x0e\x32\x31.fintekkers.models.security.bond.AuctionTypeProto\x12Q\n\x1eprice_for_single_price_auction\x18\x19 \x01(\x0b\x32).fintekkers.models.util.DecimalValueProto\x12\x41\n\x0etotal_accepted\x18\x1a \x01(\x0b\x32).fintekkers.models.util.DecimalValueProto\x12I\n\x16mature_security_amount\x18\x1b \x01(\x0b\x32).fintekkers.models.util.DecimalValueProtoB\x12\x42\x0eIssuanceProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.security.bond.issuance_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\016IssuanceProtosP\001'
  _globals['_ISSUANCEPROTO']._serialized_start=267
  _globals['_ISSUANCEPROTO']._serialized_end=1112
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/position/field.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/position/field.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&fintekkers/models/position/field.proto\x12\x1a\x66intekkers.models.position*\xa8\x04\n\nFieldProto\x12\x11\n\rUNKNOWN_FIELD\x10\x00\x12\x06\n\x02ID\x10\x01\x12\t\n\x05\x41S_OF\x10\x02\x12\x12\n\x0e\x45\x46\x46\x45\x43TIVE_DATE\x10\n\x12\x0c\n\x08STRATEGY\x10\x0b\x12\x0c\n\x08SECURITY\x10\x0c\x12\x18\n\x14SECURITY_DESCRIPTION\x10=\x12\x18\n\x14SECURITY_ISSUER_NAME\x10>\x12\x18\n\x14\x43\x41SH_IMPACT_SECURITY\x10\r\x12\x0f\n\x0b\x41SSET_CLASS\x10\x32\x12\x11\n\rPRODUCT_CLASS\x10\x33\x12\x10\n\x0cPRODUCT_TYPE\x10\x34\x12\x0f\n\x0bSECURITY_ID\x10\x35\x12\x0e\n\nIDENTIFIER\x10\x36\x12\t\n\x05TENOR\x10\x37\x12\x0e\n\nISSUE_DATE\x10:\x12\x11\n\rMATURITY_DATE\x10\x38\x12\x12\n\x0e\x41\x44JUSTED_TENOR\x10\x39\x12\r\n\tPORTFOLIO\x10\x0e\x12\x10\n\x0cPORTFOLIO_ID\x10\x0f\x12\x12\n\x0ePORTFOLIO_NAME\x10<\x12\t\n\x05PRICE\x10\x10\x12\x0c\n\x08PRICE_ID\x10\x11\x12\x10\n\x0cIS_CANCELLED\x10\x12\x12\x13\n\x0fPOSITION_STATUS\x10\x13\x12\x0e\n\nTRADE_DATE\x10\x1e\x12\x13\n\x0fSETTLEMENT_DATE\x10\x1f\x12\x14\n\x10TRANSACTION_TYPE\x10 \x12\x15\n\x11TAX_LOT_OPEN_DATE\x10(\x12\x16\n\x12TAX_LOT_CLOSE_DATE\x10)B\x0f\x42\x0b\x46ieldProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.position.field_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\013FieldProtosP\001'
  _globals['_FIELDPROTO']._serialized_start=71
  _globals['_FIELDPROTO']._serialized_end=623
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/security/security.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/security/security.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.util import decimal_value_pb2 as fintekkers_dot_models_dot_util_dot_decimal__value__pb2
from fintekkers.models.util import local_date_pb2 as fintekkers_dot_models_dot_util_dot_local__date__pb2
from fintekkers.models.util import local_timestamp_pb2 as fintekkers_dot_models_dot_util_dot_local__timestamp__pb2
from fintekkers.models.util import uuid_pb2 as fintekkers_dot_models_dot_util_dot_uuid__pb2
from fintekkers.models.security.identifier import identifier_pb2 as fintekkers_dot_models_dot_security_dot_identifier_dot_identifier__pb2
from fintekkers.models.security.bond import issuance_pb2 as fintekkers_dot_models_dot_security_dot_bond_dot_issuance__pb2
from fintekkers.models.security import security_type_pb2 as fintekkers_dot_models_dot_security_dot_security__type__pb2
from fintekkers.models.security import security_quantity_type_pb2 as fintekkers_dot_models_dot_security_dot_security__quantity__type__pb2
from fintekkers.models.security import coupon_frequency_pb2 as fintekkers_dot_models_dot_security_dot_coupon__frequency__pb2
from fintekkers.models.security import coupon_type_pb2 as fintekkers_dot_models_dot_security_dot_coupon__type__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)fintekkers/models/security/security.proto\x12\x1a\x66intekkers.models.security\x1a*fintekkers/models/util/decimal_value.proto\x1a\'fintekkers/models/util/local_date.proto\x1a,fintekkers/models/util/local_timestamp.proto\x1a!fintekkers/models/util/uuid.proto\x1a\x36\x66intekkers/models/security/identifier/identifier.proto\x1a.fintekkers/models/security/bond/issuance.proto\x1a.fintekkers/models/security/security_type.proto\x1a\x37\x66intekkers/models/security/security_quantity_type.proto\x1a\x31\x66intekkers/models/security/coupon_frequency.proto\x1a,fintekkers/models/security/coupon_type.proto\"\xac\t\n\rSecurityProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12/\n\x04uuid\x18\x05 \x01(\x0b\x32!.fintekkers.models.util.UUIDProto\x12:\n\x05\x61s_of\x18\x06 \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12\x0f\n\x07is_link\x18\x07 \x01(\x08\x12?\n\nvalid_from\x18\x08 \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12=\n\x08valid_to\x18\t \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12\x44\n\rsecurity_type\x18\n \x01(\x0e\x32-.fintekkers.models.security.SecurityTypeProto\x12\x13\n\x0b\x61sset_class\x18\x0b \x01(\t\x12\x13\n\x0bissuer_name\x18\x0c \x01(\t\x12\x46\n\x13settlement_currency\x18\r \x01(\x0b\x32).fintekkers.models.security.SecurityProto\x12L\n\rquantity_type\x18\x0e \x01(\x0e\x32\x35.fintekkers.models.security.SecurityQuantityTypeProto\x12?\n\nidentifier\x18( \x01(\x0b\x32+.fintekkers.models.security.IdentifierProto\x12\x13\n\x0b\x64\x65scription\x18) \x01(\t\x12\x0f\n\x07\x63\x61sh_id\x18\x32 \x01(\t\x12>\n\x0b\x63oupon_rate\x18< \x01(\x0b\x32).fintekkers.models.util.DecimalValueProto\x12@\n\x0b\x63oupon_type\x18= \x01(\x0e\x32+.fintekkers.models.security.CouponTypeProto\x12J\n\x10\x63oupon_frequency\x18> \x01(\x0e\x32\x30.fintekkers.models.security.CouponFrequencyProto\x12:\n\ndated_date\x18? \x01(\x0b\x32&.fintekkers.models.util.LocalDateProto\x12=\n\nface_value\x18@ \x01(\x0b\x32).fintekkers.models.util.DecimalValueProto\x12:\n\nissue_date\x18\x41 \x01(\x0b\x32&.fintekkers.models.util.LocalDateProto\x12=\n\rmaturity_date\x18\x42 \x01(\x0b\x32&.fintekkers.models.util.LocalDateProto\x12\x45\n\rissuance_info\x18\x43 \x03(\x0b\x32..fintekkers.models.security.bond.IssuanceProtoB\x12\x42\x0eSecurityProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.security.security_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\016SecurityProtosP\001'
  _globals['_SECURITYPROTO']._serialized_start=546
  _globals['_SECURITYPROTO']._serialized_end=1742
# @@protoc_insertion_point(module_scope)



from fintekkers.models.util import uuid_pb2 as _uuid_pb2
from fintekkers.models.util import local_timestamp_pb2 as _local_timestamp_pb2
from fintekkers.models.position import position_filter_pb2 as _position_filter_pb2
from fintekkers.models.util import date_range_pb2 as _date_range_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class PriceFrequencyProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    PRICE_FREQUENCY_UNSPECIFIED: _ClassVar[PriceFrequencyProto]
    PRICE_FREQUENCY_WEEKLY: _ClassVar[PriceFrequencyProto]
    PRICE_FREQUENCY_DAILY: _ClassVar[PriceFrequencyProto]
    PRICE_FREQUENCY_HOURLY: _ClassVar[PriceFrequencyProto]
    PRICE_FREQUENCY_MINUTE: _ClassVar[PriceFrequencyProto]
    PRICE_FREQUENCY_EVERY_TICK: _ClassVar[PriceFrequencyProto]

class PriceHorizonProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    PRICE_HORIZON_UNSPECIFIED: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_1_DAY: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_5_DAYS: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_1_WEEK: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_1_MONTH: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_6_MONTHS: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_1_YEAR: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_5_YEAR: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_MAX: _ClassVar[PriceHorizonProto]
    PRICE_HORIZON_YEAR_TO_DATE: _ClassVar[PriceHorizonProto]
PRICE_FREQUENCY_UNSPECIFIED: PriceFrequencyProto
PRICE_FREQUENCY_WEEKLY: PriceFrequencyProto
PRICE_FREQUENCY_DAILY: PriceFrequencyProto
PRICE_FREQUENCY_HOURLY: PriceFrequencyProto
PRICE_FREQUENCY_MINUTE: PriceFrequencyProto
PRICE_FREQUENCY_EVERY_TICK: PriceFrequencyProto
PRICE_HORIZON_UNSPECIFIED: PriceHorizonProto
PRICE_HORIZON_1_DAY: PriceHorizonProto
PRICE_HORIZON_5_DAYS: PriceHorizonProto
PRICE_HORIZON_1_WEEK: PriceHorizonProto
PRICE_HORIZON_1_MONTH: PriceHorizonProto
PRICE_HORIZON_6_MONTHS: PriceHorizonProto
PRICE_HORIZON_1_YEAR: PriceHorizonProto
PRICE_HORIZON_5_YEAR: PriceHorizonProto
PRICE_HORIZON_MAX: PriceHorizonProto
PRICE_HORIZON_YEAR_TO_DATE: PriceHorizonProto

class QueryPriceRequestProto(_message.Message):
    __slots__ = ("object_class", "version", "uuIds", "search_price_input", "as_of", "frequency", "horizon", "date_range")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    UUIDS_FIELD_NUMBER: _ClassVar[int]
    SEARCH_PRICE_INPUT_FIELD_NUMBER: _ClassVar[int]
    AS_OF_FIELD_NUMBER: _ClassVar[int]
    FREQUENCY_FIELD_NUMBER: _ClassVar[int]
    HORIZON_FIELD_NUMBER: _ClassVar[int]
    DATE_RANGE_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    uuIds: _containers.RepeatedCompositeFieldContainer[_uuid_pb2.UUIDProto]
    search_price_input: _position_filter_pb2.PositionFilterProto
    as_of: _local_timestamp_pb2.LocalTimestampProto
    frequency: PriceFrequencyProto
    horizon: PriceHorizonProto
    date_range: _date_range_pb2.DateRangeProto
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., uuIds: _Optional[_Iterable[_Union[_uuid_pb2.UUIDProto, _Mapping]]] = ..., search_price_input: _Optional[_Union[_position_filter_pb2.PositionFilterProto, _Mapping]] = ..., as_of: _Optional[_Union[_local_timestamp_pb2.LocalTimestampProto, _Mapping]] = ..., frequency: _Optional[_Union[PriceFrequencyProto, str]] = ..., horizon: _Optional[_Union[PriceHorizonProto, str]] = ..., date_range: _Optional[_Union[_date_range_pb2.DateRangeProto, _Mapping]] = ...) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class FieldProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_FIELD: _ClassVar[FieldProto]
    ID: _ClassVar[FieldProto]
    AS_OF: _ClassVar[FieldProto]
    EFFECTIVE_DATE: _ClassVar[FieldProto]
    STRATEGY: _ClassVar[FieldProto]
    SECURITY: _ClassVar[FieldProto]
    SECURITY_DESCRIPTION: _ClassVar[FieldProto]
    SECURITY_ISSUER_NAME: _ClassVar[FieldProto]
    CASH_IMPACT_SECURITY: _ClassVar[FieldProto]
    ASSET_CLASS: _ClassVar[FieldProto]
    PRODUCT_CLASS: _ClassVar[FieldProto]
    PRODUCT_TYPE: _ClassVar[FieldProto]
    SECURITY_ID: _ClassVar[FieldProto]
    IDENTIFIER: _ClassVar[FieldProto]
    TENOR: _ClassVar[FieldProto]
    ISSUE_DATE: _ClassVar[FieldProto]
    MATURITY_DATE: _ClassVar[FieldProto]
    ADJUSTED_TENOR: _ClassVar[FieldProto]
    PORTFOLIO: _ClassVar[FieldProto]
    PORTFOLIO_ID: _ClassVar[FieldProto]
    PORTFOLIO_NAME: _ClassVar[FieldProto]
    PRICE: _ClassVar[FieldProto]
    PRICE_ID: _ClassVar[FieldProto]
    IS_CANCELLED: _ClassVar[FieldProto]
    POSITION_STATUS: _ClassVar[FieldProto]
    TRADE_DATE: _ClassVar[FieldProto]
    SETTLEMENT_DATE: _ClassVar[FieldProto]
    TRANSACTION_TYPE: _ClassVar[FieldProto]
    TAX_LOT_OPEN_DATE: _ClassVar[FieldProto]
    TAX_LOT_CLOSE_DATE: _ClassVar[FieldProto]
UNKNOWN_FIELD: FieldProto
ID: FieldProto
AS_OF: FieldProto
EFFECTIVE_DATE: FieldProto
STRATEGY: FieldProto
SECURITY: FieldProto
SECURITY_DESCRIPTION: FieldProto
SECURITY_ISSUER_NAME: FieldProto
CASH_IMPACT_SECURITY: FieldProto
ASSET_CLASS: FieldProto
PRODUCT_CLASS: FieldProto
PRODUCT_TYPE: FieldProto
SECURITY_ID: FieldProto
IDENTIFIER: FieldProto
TENOR: FieldProto
ISSUE_DATE: FieldProto
MATURITY_DATE: FieldProto
ADJUSTED_TENOR: FieldProto
PORTFOLIO: FieldProto
PORTFOLIO_ID: FieldProto
PORTFOLIO_NAME: FieldProto
PRICE: FieldProto
PRICE_ID: FieldProto
IS_CANCELLED: FieldProto
POSITION_STATUS: FieldProto
TRADE_DATE: FieldProto
SETTLEMENT_DATE: FieldProto
TRANSACTION_TYPE: FieldProto
TAX_LOT_OPEN_DATE: FieldProto
TAX_LOT_CLOSE_DATE: FieldProto



from typing import Generator
from uuid import UUID
from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.util.uuid_pb2 import UUIDProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry
from fintekkers.requests.security.query_security_response_pb2 import (
    QuerySecurityResponseProto,
)
from fintekkers.services.security_service.security_service_pb2_grpc import SecurityStub
from fintekkers.models.position.position_pb2 import PositionProto

from fintekkers.wrappers.models.position import Position
from fintekkers.wrappers.models.security.security import Security
from fintekkers.wrappers.requests.security import (
    QuerySecurityRequest,
    CreateSecurityRequest,
)
from fintekkers.wrappers.services.util.Environment import EnvConfig

from fintekkers.requests.security.get_fields_response_pb2 import GetFieldsResponseProto
from fintekkers.requests.security.get_field_values_response_pb2 import GetFieldValuesResponseProto
from google.protobuf.empty_pb2 import Empty
from google.protobuf.any_pb2 import Any
from fintekkers.requests.security.get_field_values_request_pb2 import GetFieldValuesRequestProto
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil

class SecurityService:
    def __init__(self):
        print("SecurityService connecting to: " + EnvConfig.api_url())
        self.stub = SecurityStub(EnvConfig.get_channel())

    def search(self, request: QuerySecurityRequest) -> Generator[Security, None, None]:
        responses = self.stub.Search(request=request.proto)

        try:
            while not responses._is_complete():
                response: QuerySecurityResponseProto = responses.next()

                for security_proto in response.security_response:
                    yield Security(security_proto)
        except StopIteration:
            pass
        except Exception as e:
            print(e)

        # This will send the cancel message to the server to kill the connection
        responses.cancel()

    def create_or_update(self, request: CreateSecurityRequest):
        return self.stub.CreateOrUpdate(request.proto)

    def get_security_by_uuid(uuid: UUID) -> Security:
        """
        Parameters:
            A UUID

        Returns:
            request (SecurityProto): Returns the Security proto for the UUID, or None if doesn't exist
        """
        uuid_proto = UUIDProto(raw_uuid=uuid.bytes)

        request: QuerySecurityRequest = QuerySecurityRequest.create_query_request(
            {
                FieldProto.ID: uuid_proto,
            }
        )

        securities = SecurityService().search(request)

        for security in securities:
            return security

    def get_fields(self) -> list[FieldProto]:
        response:GetFieldsResponseProto = self.stub.GetFields(Empty())
        return response.fields

    def get_field_values(self, field:FieldProto) -> list[object]:
        request = GetFieldValuesRequestProto(field=field)
        response:GetFieldValuesResponseProto = self.stub.GetFieldValues(request)

        values = []
        for value in response.values:
            value:Any
            
            a: FieldMapEntry = FieldMapEntry(field=field, field_value_packed=value)

            ##TODO: This is a hack to get the field value. We need to find a better way to do this. 
            # Instead we should extract the logic relating to FieldMapEntry from Position
            # to make it reusable without having to create a Position object.
            position = Position(positionProto=PositionProto(fields=[a]))
            field_value = position.get_field(a)
            values.append(field_value)

        return values



import time
from datetime import date, datetime
from uuid import uuid4
from google.protobuf.any_pb2 import Any
from google.protobuf.timestamp_pb2 import Timestamp

from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.position.position_filter_pb2 import PositionFilterProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry
from fintekkers.models.price.price_pb2 import PriceProto
from fintekkers.models.security.coupon_frequency_pb2 import NO_COUPON, SEMIANNUALLY
from fintekkers.models.security.coupon_type_pb2 import FIXED, FLOAT, ZERO
from fintekkers.models.security.identifier.identifier_pb2 import IdentifierProto
from fintekkers.models.security.identifier.identifier_type_pb2 import CUSIP
from fintekkers.models.security.security_pb2 import SecurityProto
from fintekkers.models.security.security_quantity_type_pb2 import ORIGINAL_FACE_VALUE
from fintekkers.models.security.security_type_pb2 import (
    BOND_SECURITY,
    FRN,
    SecurityTypeProto,
)
from fintekkers.models.util.date_range_pb2 import DateRangeProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.models.util.uuid_pb2 import UUIDProto

from fintekkers.models.security.bond.issuance_pb2 import IssuanceProto

from fintekkers.requests.price.create_price_request_pb2 import CreatePriceRequestProto
from fintekkers.requests.price.query_price_request_pb2 import PriceFrequencyProto, PriceHorizonProto, QueryPriceRequestProto
from fintekkers.requests.security.create_security_request_pb2 import (
    CreateSecurityRequestProto,
)
from fintekkers.requests.security.query_security_request_pb2 import (
    QuerySecurityRequestProto,
)

from fintekkers.wrappers.models.security.security import Security
from fintekkers.wrappers.models.util.date_utils import get_date_proto
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil


class CreatePriceRequest:
    @staticmethod
    def create_price_request(
        security: SecurityProto,
        price: float = 0.0,
    ):
        """
        Creates a request to create a price for the current asOf date with the provided price

            Parameters:
                    Security: the Security Proto for this price
                    price: the float value of the price

            Returns:
                    request (CreatePriceRequest): A request wrapper, with the fields attached
        """
        return CreatePriceRequest.create_or_update_request(security, price)

    @staticmethod
    def create_or_update_request(security: Security, price:float):
        """TODO: Should we look up first?"""
        as_of_proto: LocalTimestampProto = ProtoSerializationUtil.serialize(
            datetime.now()
        )

        proto: CreatePriceRequestProto = CreatePriceRequestProto(
            create_price_input=PriceProto(
                as_of=as_of_proto,
                valid_from=as_of_proto,
                price=ProtoSerializationUtil.serialize(price),
                security=security
            )
        )

        return CreatePriceRequest(proto=proto)

    def __init__(self, proto: CreateSecurityRequestProto):
        self.proto = proto


class QueryPriceRequest:
    def __init__(self, proto: CreatePriceRequestProto):
        self.proto = proto

    @staticmethod
    def create_query_request(fields: dict, 
                             frequency: PriceFrequencyProto, 
                             horizon: PriceHorizonProto):
        """
        Returns a query request from a dict of field/values

                Parameters:
                        fields (dict): A dictionary of fields with values

                Returns:
                        request (QuerySecurityRequest): A request wrapper, with the fields attached. It assumes that all filters are an equals operation
        """

        filters = []

        for field in fields:
            field: FieldProto

            field_value = fields[field]

            if isinstance(field_value, str):
                entry = FieldMapEntry(field=field, string_value=field_value)
            else:
                packed_value: Any = Any()
                packed_value.Pack(msg=field_value)
                entry = FieldMapEntry(field=field, field_value_packed=packed_value)

            filters.append(entry)

        as_of_proto: LocalTimestampProto = ProtoSerializationUtil.serialize(
            datetime.now()
        )

        request: QuerySecurityRequestProto = QueryPriceRequestProto(
            search_price_input=PositionFilterProto(filters=filters),
            as_of=as_of_proto,
            frequency=frequency,
            horizon=horizon
        )

        return QueryPriceRequest(proto=request)


    @staticmethod
    def create_query_request(fields: dict,
                             frequency:PriceFrequencyProto,
                             start_date: date,
                             end_date: date):
        """
        Returns a query request from a dict of field/values

                Parameters:
                        fields (dict): A dictionary of fields with values

                Returns:
                        request (QuerySecurityRequest): A request wrapper, with the fields attached. It assumes that all filters are an equals operation
        """

        filters = []

        for field in fields:
            field: FieldProto

            field_value = fields[field]

            if isinstance(field_value, str):
                entry = FieldMapEntry(field=field, string_value=field_value)
            else:
                packed_value: Any = Any()
                packed_value.Pack(msg=field_value)
                entry = FieldMapEntry(field=field, field_value_packed=packed_value)

            filters.append(entry)

        as_of_proto: LocalTimestampProto = ProtoSerializationUtil.serialize(
            datetime.now()
        )

        start_date_proto: LocalTimestampProto = ProtoSerializationUtil.serialize(start_date) if start_date else None
        end_date_proto: LocalTimestampProto = ProtoSerializationUtil.serialize(end_date) if end_date else None

        date_range_proto: DateRangeProto = DateRangeProto(start_date=start_date_proto, end_date=end_date_proto) if start_date and end_date else None

        request: QuerySecurityRequestProto = QueryPriceRequestProto(
            search_price_input=PositionFilterProto(filters=filters),
            as_of=as_of_proto,
            frequency=frequency,
            date_range=date_range_proto
        )

        return QueryPriceRequest(proto=request)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/position/position_status.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/position/position_status.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0fintekkers/models/position/position_status.proto\x12\x1a\x66intekkers.models.position*P\n\x13PositionStatusProto\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x10\n\x0cHYPOTHETICAL\x10\x01\x12\x0c\n\x08INTENDED\x10\x02\x12\x0c\n\x08\x45XECUTED\x10\x03\x42\x18\x42\x14PositionStatusProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.position.position_status_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\024PositionStatusProtosP\001'
  _globals['_POSITIONSTATUSPROTO']._serialized_start=80
  _globals['_POSITIONSTATUSPROTO']._serialized_end=160
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from fintekkers.models.util.lock import node_partition_pb2 as fintekkers_dot_models_dot_util_dot_lock_dot_node__partition__pb2
from fintekkers.models.util.lock import node_state_pb2 as fintekkers_dot_models_dot_util_dot_lock_dot_node__state__pb2
from fintekkers.requests.util.lock import lock_request_pb2 as fintekkers_dot_requests_dot_util_dot_lock_dot_lock__request__pb2
from fintekkers.requests.util.lock import lock_response_pb2 as fintekkers_dot_requests_dot_util_dot_lock_dot_lock__response__pb2
from fintekkers.services.lock_service import lock_service_pb2 as fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

GRPC_GENERATED_VERSION = '1.73.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in fintekkers/services/lock_service/lock_service_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class LockStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ClaimLock = channel.unary_unary(
                '/fintekkers.services.lock_service.Lock/ClaimLock',
                request_serializer=fintekkers_dot_requests_dot_util_dot_lock_dot_lock__request__pb2.LockRequestProto.SerializeToString,
                response_deserializer=fintekkers_dot_requests_dot_util_dot_lock_dot_lock__response__pb2.LockResponseProto.FromString,
                _registered_method=True)
        self.SubscribeToLockUpdates = channel.unary_stream(
                '/fintekkers.services.lock_service.Lock/SubscribeToLockUpdates',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=fintekkers_dot_models_dot_util_dot_lock_dot_node__state__pb2.NodeState.FromString,
                _registered_method=True)
        self.CreateNamespace = channel.unary_unary(
                '/fintekkers.services.lock_service.Lock/CreateNamespace',
                request_serializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.CreateNamespaceRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                _registered_method=True)
        self.CreatePartition = channel.unary_unary(
                '/fintekkers.services.lock_service.Lock/CreatePartition',
                request_serializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.CreatePartitionRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                _registered_method=True)
        self.ListNamespaces = channel.unary_unary(
                '/fintekkers.services.lock_service.Lock/ListNamespaces',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.FromString,
                _registered_method=True)
        self.ListPartitions = channel.unary_unary(
                '/fintekkers.services.lock_service.Lock/ListPartitions',
                request_serializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.SerializeToString,
                response_deserializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.PartitionsList.FromString,
                _registered_method=True)
        self.GetAllPartitionStatus = channel.unary_unary(
                '/fintekkers.services.lock_service.Lock/GetAllPartitionStatus',
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NodeStateList.FromString,
                _registered_method=True)
        self.GetAllPartitionStatusForNamespaces = channel.unary_unary(
                '/fintekkers.services.lock_service.Lock/GetAllPartitionStatusForNamespaces',
                request_serializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.SerializeToString,
                response_deserializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NodeStateList.FromString,
                _registered_method=True)
        self.GetPartitionStatus = channel.unary_unary(
                '/fintekkers.services.lock_service.Lock/GetPartitionStatus',
                request_serializer=fintekkers_dot_models_dot_util_dot_lock_dot_node__partition__pb2.NodePartition.SerializeToString,
                response_deserializer=fintekkers_dot_models_dot_util_dot_lock_dot_node__state__pb2.NodeState.FromString,
                _registered_method=True)


class LockServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ClaimLock(self, request, context):
        """Allows a Fintekkers service to claim the lock for a partition. 
        See {fintekkers.request.util.lock.LockRequestProto} for details
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SubscribeToLockUpdates(self, request, context):
        """Streams any change in lock owner for any namespace/partition to the subscriber. 
        Heartbeat updates are not streamed to subscribers. If a subsciber wants to build an in-memory cache of parition state
        they should first subscribe to lock updates, then query the G
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreateNamespace(self, request, context):
        """Create a namespace
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CreatePartition(self, request, context):
        """Create a partition
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListNamespaces(self, request, context):
        """Lists the possible namespaces
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListPartitions(self, request, context):
        """Lists all partitions for the given list of namespaces
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllPartitionStatus(self, request, context):
        """Returns the current status of all nodes, across all namespaces and partitions.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllPartitionStatusForNamespaces(self, request, context):
        """Returns the current status of all nodes, across all namespaces and partitions.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPartitionStatus(self, request, context):
        """In namespace / parition
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_LockServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ClaimLock': grpc.unary_unary_rpc_method_handler(
                    servicer.ClaimLock,
                    request_deserializer=fintekkers_dot_requests_dot_util_dot_lock_dot_lock__request__pb2.LockRequestProto.FromString,
                    response_serializer=fintekkers_dot_requests_dot_util_dot_lock_dot_lock__response__pb2.LockResponseProto.SerializeToString,
            ),
            'SubscribeToLockUpdates': grpc.unary_stream_rpc_method_handler(
                    servicer.SubscribeToLockUpdates,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=fintekkers_dot_models_dot_util_dot_lock_dot_node__state__pb2.NodeState.SerializeToString,
            ),
            'CreateNamespace': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateNamespace,
                    request_deserializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.CreateNamespaceRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'CreatePartition': grpc.unary_unary_rpc_method_handler(
                    servicer.CreatePartition,
                    request_deserializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.CreatePartitionRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'ListNamespaces': grpc.unary_unary_rpc_method_handler(
                    servicer.ListNamespaces,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.SerializeToString,
            ),
            'ListPartitions': grpc.unary_unary_rpc_method_handler(
                    servicer.ListPartitions,
                    request_deserializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.FromString,
                    response_serializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.PartitionsList.SerializeToString,
            ),
            'GetAllPartitionStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllPartitionStatus,
                    request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                    response_serializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NodeStateList.SerializeToString,
            ),
            'GetAllPartitionStatusForNamespaces': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllPartitionStatusForNamespaces,
                    request_deserializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.FromString,
                    response_serializer=fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NodeStateList.SerializeToString,
            ),
            'GetPartitionStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPartitionStatus,
                    request_deserializer=fintekkers_dot_models_dot_util_dot_lock_dot_node__partition__pb2.NodePartition.FromString,
                    response_serializer=fintekkers_dot_models_dot_util_dot_lock_dot_node__state__pb2.NodeState.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'fintekkers.services.lock_service.Lock', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('fintekkers.services.lock_service.Lock', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class Lock(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ClaimLock(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/ClaimLock',
            fintekkers_dot_requests_dot_util_dot_lock_dot_lock__request__pb2.LockRequestProto.SerializeToString,
            fintekkers_dot_requests_dot_util_dot_lock_dot_lock__response__pb2.LockResponseProto.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SubscribeToLockUpdates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/SubscribeToLockUpdates',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            fintekkers_dot_models_dot_util_dot_lock_dot_node__state__pb2.NodeState.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreateNamespace(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/CreateNamespace',
            fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.CreateNamespaceRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CreatePartition(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/CreatePartition',
            fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.CreatePartitionRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListNamespaces(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/ListNamespaces',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListPartitions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/ListPartitions',
            fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.SerializeToString,
            fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.PartitionsList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetAllPartitionStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/GetAllPartitionStatus',
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NodeStateList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetAllPartitionStatusForNamespaces(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/GetAllPartitionStatusForNamespaces',
            fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NamespaceList.SerializeToString,
            fintekkers_dot_services_dot_lock__service_dot_lock__service__pb2.NodeStateList.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPartitionStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/fintekkers.services.lock_service.Lock/GetPartitionStatus',
            fintekkers_dot_models_dot_util_dot_lock_dot_node__partition__pb2.NodePartition.SerializeToString,
            fintekkers_dot_models_dot_util_dot_lock_dot_node__state__pb2.NodeState.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



from google.protobuf.any_pb2 import Any
from datetime import datetime

from fintekkers.models.portfolio.portfolio_pb2 import PortfolioProto
from fintekkers.models.position.field_pb2 import FieldProto
from fintekkers.models.position.position_filter_pb2 import PositionFilterProto
from fintekkers.models.position.position_util_pb2 import FieldMapEntry
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto

from fintekkers.requests.portfolio.create_portfolio_request_pb2 import (
    CreatePortfolioRequestProto,
)
from fintekkers.requests.portfolio.query_portfolio_request_pb2 import (
    QueryPortfolioRequestProto,
)

from fintekkers.wrappers.models.portfolio import Portfolio
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil


class CreatePortfolioRequest:
    @staticmethod
    def create_portfolio_request_from_name(
        portfolio_name: str,
    ):
        return CreatePortfolioRequestProto(
            create_portfolio_input=Portfolio.create_portfolio(portfolio_name).proto
        )

    @staticmethod
    def create_portfolio_request_from_proto(
        portfolio: PortfolioProto,
    ):
        return CreatePortfolioRequestProto(
            create_portfolio_input=portfolio
        )


class QueryPortfolioRequest:
    @staticmethod
    def create_query_request(fields: dict):
        """
        Returns a query request from a dict of field/values

                Parameters:
                        fields (dict): A dictionary of fields with values

                Returns:
                        request (CreateTransactionRequest): A request wrapper, with the fields attached. It assumes that all filters are an equals operation
        """

        filters = []

        # TODO: This code is duplicated in transaction.py
        for field in fields:
            field: FieldProto

            field_value = fields[field]
            entry = None

            if field_value.__class__ == str:
                entry = FieldMapEntry(field=field, string_value=field_value)
            else:
                packed_value = Any()
                packed_value.Pack(msg=field_value)

                entry = FieldMapEntry(field=field, field_value_packed=packed_value)

            filters.append(entry)

        as_of_proto: LocalTimestampProto = ProtoSerializationUtil.serialize(
            datetime.now()
        )

        query_request: QueryPortfolioRequestProto = QueryPortfolioRequestProto(
            search_portfolio_input=PositionFilterProto(filters=filters),
            as_of=as_of_proto,
        )

        return QueryPortfolioRequest(query_request)

    def __init__(self, proto: QueryPortfolioRequestProto):
        self.proto = proto



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/services/valuation-service/valuation_service.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/services/valuation-service/valuation_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.requests.valuation import valuation_request_pb2 as fintekkers_dot_requests_dot_valuation_dot_valuation__request__pb2
from fintekkers.requests.valuation import valuation_response_pb2 as fintekkers_dot_requests_dot_valuation_dot_valuation__response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=fintekkers/services/valuation-service/valuation_service.proto\x12%fintekkers.services.valuation_service\x1a\x35\x66intekkers/requests/valuation/valuation_request.proto\x1a\x36\x66intekkers/requests/valuation/valuation_response.proto2\x88\x01\n\tValuation\x12{\n\x0cRunValuation\x12\x34.fintekkers.requests.valuation.ValuationRequestProto\x1a\x35.fintekkers.requests.valuation.ValuationResponseProtoB\x06\x88\x01\x01\x90\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.services.valuation_service.valuation_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\210\001\001\220\001\001'
  _globals['_VALUATION']._serialized_start=216
  _globals['_VALUATION']._serialized_end=352
_builder.BuildServices(DESCRIPTOR, 'fintekkers.services.valuation_service.valuation_service_pb2', _globals)
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class SecurityTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_SECURITY_TYPE: _ClassVar[SecurityTypeProto]
    CASH_SECURITY: _ClassVar[SecurityTypeProto]
    EQUITY_SECURITY: _ClassVar[SecurityTypeProto]
    BOND_SECURITY: _ClassVar[SecurityTypeProto]
    TIPS: _ClassVar[SecurityTypeProto]
    FRN: _ClassVar[SecurityTypeProto]
UNKNOWN_SECURITY_TYPE: SecurityTypeProto
CASH_SECURITY: SecurityTypeProto
EQUITY_SECURITY: SecurityTypeProto
BOND_SECURITY: SecurityTypeProto
TIPS: SecurityTypeProto
FRN: SecurityTypeProto



from fintekkers.models.portfolio import portfolio_pb2 as _portfolio_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CreatePortfolioRequestProto(_message.Message):
    __slots__ = ("object_class", "version", "create_portfolio_input")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    CREATE_PORTFOLIO_INPUT_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    create_portfolio_input: _portfolio_pb2.PortfolioProto
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., create_portfolio_input: _Optional[_Union[_portfolio_pb2.PortfolioProto, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/util/lock/lock_request.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/util/lock/lock_request.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.util.lock import node_partition_pb2 as fintekkers_dot_models_dot_util_dot_lock_dot_node__partition__pb2
from fintekkers.models.util import endpoint_pb2 as fintekkers_dot_models_dot_util_dot_endpoint__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0fintekkers/requests/util/lock/lock_request.proto\x12\x1d\x66intekkers.requests.util.lock\x1a\x30\x66intekkers/models/util/lock/node_partition.proto\x1a%fintekkers/models/util/endpoint.proto\"\xb1\x01\n\x10LockRequestProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x42\n\x0enode_partition\x18\x0b \x01(\x0b\x32*.fintekkers.models.util.lock.NodePartition\x12\x32\n\x08\x65ndpoint\x18\x0c \x01(\x0b\x32 .fintekkers.models.util.EndpointB\x15\x42\x11LockRequestProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.util.lock.lock_request_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\021LockRequestProtosP\001'
  _globals['_LOCKREQUESTPROTO']._serialized_start=173
  _globals['_LOCKREQUESTPROTO']._serialized_end=350
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/price/create_price_request.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/price/create_price_request.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.price import price_pb2 as fintekkers_dot_models_dot_price_dot_price__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4fintekkers/requests/price/create_price_request.proto\x12\x19\x66intekkers.requests.price\x1a#fintekkers/models/price/price.proto\"\x81\x01\n\x17\x43reatePriceRequestProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12?\n\x12\x63reate_price_input\x18\x14 \x01(\x0b\x32#.fintekkers.models.price.PriceProtoB\x1c\x42\x18\x43reatePriceRequestProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.price.create_price_request_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\030CreatePriceRequestProtosP\001'
  _globals['_CREATEPRICEREQUESTPROTO']._serialized_start=121
  _globals['_CREATEPRICEREQUESTPROTO']._serialized_end=250
# @@protoc_insertion_point(module_scope)



from fintekkers.requests.util.errors import error_pb2 as _error_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class SummaryProto(_message.Message):
    __slots__ = ("errors", "warnings")
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    WARNINGS_FIELD_NUMBER: _ClassVar[int]
    errors: _containers.RepeatedCompositeFieldContainer[_error_pb2.ErrorProto]
    warnings: _containers.RepeatedCompositeFieldContainer[_error_pb2.WarningProto]
    def __init__(self, errors: _Optional[_Iterable[_Union[_error_pb2.ErrorProto, _Mapping]]] = ..., warnings: _Optional[_Iterable[_Union[_error_pb2.WarningProto, _Mapping]]] = ...) -> None: ...



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class CouponTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_COUPON_TYPE: _ClassVar[CouponTypeProto]
    FIXED: _ClassVar[CouponTypeProto]
    FLOAT: _ClassVar[CouponTypeProto]
    ZERO: _ClassVar[CouponTypeProto]
UNKNOWN_COUPON_TYPE: CouponTypeProto
FIXED: CouponTypeProto
FLOAT: CouponTypeProto
ZERO: CouponTypeProto



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/valuation/valuation_response.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/valuation/valuation_response.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.position import position_util_pb2 as fintekkers_dot_models_dot_position_dot_position__util__pb2
from fintekkers.requests.valuation import valuation_request_pb2 as fintekkers_dot_requests_dot_valuation_dot_valuation__request__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6fintekkers/requests/valuation/valuation_response.proto\x12\x1d\x66intekkers.requests.valuation\x1a.fintekkers/models/position/position_util.proto\x1a\x35\x66intekkers/requests/valuation/valuation_request.proto\"\xd6\x01\n\x16ValuationResponseProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12O\n\x11valuation_request\x18\x14 \x01(\x0b\x32\x34.fintekkers.requests.valuation.ValuationRequestProto\x12\x44\n\x0fmeasure_results\x18\x1e \x03(\x0b\x32+.fintekkers.models.position.MeasureMapEntryB\x1b\x42\x17ValuationResponseProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.valuation.valuation_response_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\027ValuationResponseProtosP\001'
  _globals['_VALUATIONRESPONSEPROTO']._serialized_start=193
  _globals['_VALUATIONRESPONSEPROTO']._serialized_end=407
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class RequestOperationTypeProto(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNKNOWN_OPERATION: _ClassVar[RequestOperationTypeProto]
    VALIDATE: _ClassVar[RequestOperationTypeProto]
    CREATE: _ClassVar[RequestOperationTypeProto]
    GET: _ClassVar[RequestOperationTypeProto]
    SEARCH: _ClassVar[RequestOperationTypeProto]
UNKNOWN_OPERATION: RequestOperationTypeProto
VALIDATE: RequestOperationTypeProto
CREATE: RequestOperationTypeProto
GET: RequestOperationTypeProto
SEARCH: RequestOperationTypeProto



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/util/endpoint.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/util/endpoint.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%fintekkers/models/util/endpoint.proto\x12\x16\x66intekkers.models.util\"A\n\x08\x45ndpoint\x12\n\n\x02ip\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\r\x12\x1b\n\x13\x66ully_qualified_url\x18\x03 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.util.endpoint_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_ENDPOINT']._serialized_start=65
  _globals['_ENDPOINT']._serialized_end=130
# @@protoc_insertion_point(module_scope)



from google.protobuf import any_pb2 as _any_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GetFieldValuesResponseProto(_message.Message):
    __slots__ = ("object_class", "version", "values")
    OBJECT_CLASS_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    object_class: str
    version: str
    values: _containers.RepeatedCompositeFieldContainer[_any_pb2.Any]
    def __init__(self, object_class: _Optional[str] = ..., version: _Optional[str] = ..., values: _Optional[_Iterable[_Union[_any_pb2.Any, _Mapping]]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/models/util/date_range.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/models/util/date_range.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.util import local_timestamp_pb2 as fintekkers_dot_models_dot_util_dot_local__timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'fintekkers/models/util/date_range.proto\x12\x16\x66intekkers.models.util\x1a,fintekkers/models/util/local_timestamp.proto\"\xad\x01\n\x0e\x44\x61teRangeProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12:\n\x05start\x18\n \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProto\x12\x38\n\x03\x65nd\x18\x0b \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.models.util.date_range_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_DATERANGEPROTO']._serialized_start=114
  _globals['_DATERANGEPROTO']._serialized_end=287
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/position/query_position_request.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/position/query_position_request.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fintekkers.models.position import field_pb2 as fintekkers_dot_models_dot_position_dot_field__pb2
from fintekkers.models.position import measure_pb2 as fintekkers_dot_models_dot_position_dot_measure__pb2
from fintekkers.models.position import position_pb2 as fintekkers_dot_models_dot_position_dot_position__pb2
from fintekkers.models.position import position_filter_pb2 as fintekkers_dot_models_dot_position_dot_position__filter__pb2
from fintekkers.models.util import local_timestamp_pb2 as fintekkers_dot_models_dot_util_dot_local__timestamp__pb2
from fintekkers.requests.util import operation_pb2 as fintekkers_dot_requests_dot_util_dot_operation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9fintekkers/requests/position/query_position_request.proto\x12\x1c\x66intekkers.requests.position\x1a&fintekkers/models/position/field.proto\x1a(fintekkers/models/position/measure.proto\x1a)fintekkers/models/position/position.proto\x1a\x30\x66intekkers/models/position/position_filter.proto\x1a,fintekkers/models/util/local_timestamp.proto\x1a(fintekkers/requests/util/operation.proto\"\x93\x04\n\x19QueryPositionRequestProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12K\n\x0eoperation_type\x18\n \x01(\x0e\x32\x33.fintekkers.requests.util.RequestOperationTypeProto\x12\x44\n\rposition_type\x18\x14 \x01(\x0e\x32-.fintekkers.models.position.PositionTypeProto\x12\x44\n\rposition_view\x18\x15 \x01(\x0e\x32-.fintekkers.models.position.PositionViewProto\x12\x36\n\x06\x66ields\x18\x1e \x03(\x0e\x32&.fintekkers.models.position.FieldProto\x12:\n\x08measures\x18\x1f \x03(\x0e\x32(.fintekkers.models.position.MeasureProto\x12\x46\n\rfilter_fields\x18  \x01(\x0b\x32/.fintekkers.models.position.PositionFilterProto\x12:\n\x05\x61s_of\x18! \x01(\x0b\x32+.fintekkers.models.util.LocalTimestampProtoB\x1e\x42\x1aQueryPositionRequestProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.position.query_position_request_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\032QueryPositionRequestProtosP\001'
  _globals['_QUERYPOSITIONREQUESTPROTO']._serialized_start=355
  _globals['_QUERYPOSITIONREQUESTPROTO']._serialized_end=886
# @@protoc_insertion_point(module_scope)



import grpc
import os
from enum import Enum

# Load environment variables from .env file
from dotenv import load_dotenv

load_dotenv()


class ServiceType(Enum):
    SECURITY_SERVICE = "8082"
    LEDGER_SERVICE = "8082"
    TRANSACTION_SERVICE = "8082"
    PORTFOLIO_SERVICE = "8082"
    VALUATION_SERVICE = "8080"
    PRICE_SERVICE = "8083"


class EnvConfig:
    default_api_url = "api.fintekkers.org"
    
    @staticmethod
    def get_env_var(key, default=None):
        value = os.environ.get(key)
        if value is None:
            if default is None:
                raise ValueError(f"Environment variable {key} is not set.")
            return default
        return value

    @staticmethod
    def api_key():
        raise NotImplementedError("API keys not supported currently.")
        # return EnvConfig.get_env_var('API_KEY')

    @staticmethod
    def api_url(service_type: ServiceType = ServiceType.LEDGER_SERVICE):
        base_url = EnvConfig.get_env_var('API_URL', EnvConfig.default_api_url)
        return f"{base_url}:{service_type.value}"

    @staticmethod
    def get_channel(service_type: ServiceType = ServiceType.LEDGER_SERVICE) -> grpc.Channel:
        url = EnvConfig.api_url(service_type)

        if "localhost" in url or "127.0.0.1" in url:
            return grpc.insecure_channel(url)
        else:
            return grpc.secure_channel(url, grpc.ssl_channel_credentials())



from fintekkers.models.position.position_status_pb2 import *
from fintekkers.models.security.identifier.identifier_pb2 import IdentifierProto
from fintekkers.models.security.tenor_pb2 import TenorProto
from fintekkers.models.util.local_date_pb2 import LocalDateProto
from fintekkers.models.util.local_timestamp_pb2 import LocalTimestampProto
from fintekkers.models.util.uuid_pb2 import UUIDProto
from fintekkers.models.util.decimal_value_pb2 import DecimalValueProto
from fintekkers.wrappers.models.security.security_identifier import Identifier
from fintekkers.wrappers.models.security.tenor import Tenor

from fintekkers.wrappers.models.transaction import TransactionType
from fintekkers.wrappers.models.util.fintekkers_uuid import FintekkersUuid

from google.protobuf.timestamp_pb2 import Timestamp
from datetime import date, datetime
import time
from pytz import timezone

from fintekkers.models.transaction.transaction_type_pb2 import TransactionTypeProto
from uuid import UUID, uuid4

from google._upb._message import EnumValueDescriptor
from google.protobuf.wrappers_pb2 import StringValue
from google.protobuf.any_pb2 import Any
class ProtoEnum:
    def __init__(self, enum_descriptor: EnumValueDescriptor, enum_value: int):
        self.enum_value: int = enum_value
        self.get_enum_descriptor = enum_descriptor
        # Will be the proto
        self.enum = ProtoEnum.get_field_descriptor(enum_descriptor)

    @staticmethod
    def from_enum_name(enum_name: str, enum_value: int):
        return ProtoEnum(
            ProtoEnum.get_field_descriptor_from_name(enum_name), enum_value
        )

    @staticmethod
    def get_field_descriptor(enum_descriptor: str):
        return ProtoEnum.get_field_descriptor_from_name(enum_descriptor.name)

    @staticmethod
    def get_field_descriptor_from_name(enum_name: str):
        if enum_name == "TRANSACTION_TYPE":
            return TransactionTypeProto
        if enum_name == "POSITION_STATUS":
            return PositionStatusProto

        raise ValueError(f"Enum has not been mapped: {enum_name}")

    def get_enum_name(self) -> str:
        """
        The string name of the enum
        """
        return self.enum_descriptor.name

    def get_enum_value(self) -> int:
        """
        Returns the numer of the enum value. Eg. zero generally means unknown,
        and 1 would be the first value in that enum."""
        return self.enum_value

    def get_enum_value_name(self) -> str:
        """
        Returns the string name of the enum, e.g. if the value is zero then
        this will generally be "UNKNOWN".
        """
        return self.enum.DESCRIPTOR.values_by_number[self.enum_value].name

    def __str__(self) -> str:
        return self.get_enum_value_name()


class ProtoSerializationUtil:
    @staticmethod
    def serialize(obj):
        if isinstance(obj, UUID):
            return UUIDProto(raw_uuid=obj.bytes)
        if isinstance(obj, Tenor):
            obj:Tenor
            return obj.as_proto()
        if type(obj) is date:
            return LocalDateProto(year=obj.year, month=obj.month, day=obj.day)
        if isinstance(obj, datetime):
            seconds_in_float: float = time.mktime(obj.timetuple())
            timestamp = Timestamp(
                seconds=int(seconds_in_float), nanos=int(obj.microsecond / 1000)
            )

            time_zone = (
                obj.timetz().tzinfo.zone
                if obj.timetz().tzinfo is not None
                else "America/New_York"
            )
            return LocalTimestampProto(timestamp=timestamp, time_zone=time_zone)
        if isinstance(obj, Identifier) or isinstance(obj, TransactionType):
            return obj.proto
        if isinstance(obj, float) or isinstance(obj, int):
            return DecimalValueProto(arbitrary_precision_value=str(obj))
        
        if isinstance(obj, str):
            return StringValue(value=obj)

        raise ValueError(
            f"Could not serialize object of type {obj.__class__.__name__}. Value: {obj}"
        )

    @staticmethod
    def deserialize(obj):
        if isinstance(obj, Any):
            #unpack a string value
            return StringValue.FromString(obj.value).value
        if isinstance(obj, UUIDProto):
            return FintekkersUuid.from_bytes(raw_uuid=obj.raw_uuid)
        if isinstance(obj, LocalDateProto):
            return date(year=obj.year, month=obj.month, day=obj.day)
        if isinstance(obj, LocalTimestampProto):
            return datetime.fromtimestamp(
                obj.timestamp.seconds, timezone(obj.time_zone)
            )
        if isinstance(obj, TenorProto):
            return Tenor(obj.tenor_type, obj.term_value)
        if isinstance(obj, IdentifierProto):
            return Identifier(obj)
        if isinstance(obj, DecimalValueProto):
            obj: DecimalValueProto
            return float(obj.arbitrary_precision_value)
        if (
            hasattr(obj, "enum_name")
            and getattr(obj, "enum_name") == "TRANSACTION_TYPE"
        ):
            return TransactionType(obj.enum_value)

        raise ValueError(
            f"Could not deserialize object of type {obj.__class__.__name__}. Value: {obj}"
        )




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: fintekkers/requests/security/get_field_values_response.proto
# Protobuf Python Version: 6.31.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    0,
    '',
    'fintekkers/requests/security/get_field_values_response.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<fintekkers/requests/security/get_field_values_response.proto\x12\x1c\x66intekkers.requests.security\x1a\x19google/protobuf/any.proto\"j\n\x1bGetFieldValuesResponseProto\x12\x14\n\x0cobject_class\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12$\n\x06values\x18\n \x03(\x0b\x32\x14.google.protobuf.AnyB B\x1cGetFieldValuesResponseProtosP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fintekkers.requests.security.get_field_values_response_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'B\034GetFieldValuesResponseProtosP\001'
  _globals['_GETFIELDVALUESRESPONSEPROTO']._serialized_start=121
  _globals['_GETFIELDVALUESRESPONSEPROTO']._serialized_end=227
# @@protoc_insertion_point(module_scope)



from fintekkers.models.security.identifier.identifier_pb2 import IdentifierProto
from fintekkers.models.security.security_pb2 import SecurityProto

from fintekkers.models.position.field_pb2 import *
from fintekkers.models.position.measure_pb2 import MeasureProto

from uuid import UUID
from datetime import datetime
from fintekkers.models.security.security_type_pb2 import SecurityTypeProto
from fintekkers.wrappers.models.security.security_identifier import Identifier

from fintekkers.wrappers.models.util.fintekkers_uuid import FintekkersUuid
from fintekkers.wrappers.models.util.serialization import ProtoSerializationUtil

class IFinancialModelObject:
    def get_field(field:FieldProto) -> object:
        pass

    def get_measure(measure:MeasureProto) -> object:
        pass

    def get_fields() -> set[FieldProto]:
        pass

    def get_measures() -> set[MeasureProto]:
        pass

    def get_as_of() -> datetime:
        pass

class RawDataModelObject:
    def __init__(self, id: UUID, as_of: datetime):
        self.id = id
        self.as_of = as_of

class Security():
    def __init__(self, proto:SecurityProto):
        self.proto:SecurityProto = proto

    def __str__(self) -> str:
        return f"ID[{str(self.get_id())}], {self.get_security_id()}[{self.proto.issuer_name}]"

    def get_fields(self) -> list[FieldProto]:
        return [
            ID, SECURITY_ID, AS_OF, ASSET_CLASS, IDENTIFIER
        ]

    def get_field(self, field:FieldProto) -> object:
        if field in (ID, SECURITY_ID):
            return self.get_id()
        elif field == AS_OF:
            return self.get_as_of()
        elif field == ASSET_CLASS:
            return self.get_asset_class()
        elif field == PRODUCT_CLASS:
            return self.get_product_class()
        elif field == PRODUCT_TYPE:
            return self.get_product_type()
        elif field == IDENTIFIER:
            return self.get_security_id()
        elif field in (TENOR, ADJUSTED_TENOR):
            return self.get_tenor()
        elif field == MATURITY_DATE:
            return self.get_maturity_date()
        elif field == ISSUE_DATE:
            return self.get_issue_date()
        else:
            raise ValueError(f"Field not mapped in Security wrapper: {FieldProto.DESCRIPTOR.values_by_number[field].name}")

    def get_id(self) -> UUID:
        uuid:FintekkersUuid = ProtoSerializationUtil.deserialize(self.proto.uuid)
        return uuid.uuid
    
    def get_as_of(self) -> datetime:
        as_of:datetime = ProtoSerializationUtil.deserialize(self.proto.as_of)
        return as_of
        
    def get_asset_class(self) -> str:
        return self.proto.asset_class
    
    def get_product_class(self) -> str:
        raise ValueError("Not implemented yet. See Java implementation for reference")
    
    def get_product_type(self) -> object:
        raise ValueError("Not implemented yet. See Java implementation for reference")
    
    def get_security_id(self) -> Identifier:
        id:IdentifierProto = self.proto.identifier
        return Identifier(id)
    
    def get_issue_date(self) -> datetime:
        return ProtoSerializationUtil.deserialize(self.proto.issue_date)

    def get_maturity_date(self) -> datetime:
        return ProtoSerializationUtil.deserialize(self.proto.maturity_date)
    
    def get_tenor(self) -> str:
        return ProtoSerializationUtil.deserialize(self.proto.tenor)

    def get_security_type(self) -> SecurityTypeProto:
        return self.proto.security_type

    def get_description(self) -> str:
        return self.proto.description

    def __str__(self):
        return f'ID[{str(self.get_security_id())}], {type(self).__name__}[{self.proto.issuer_name}]'

    def __eq__(self, other):
        if isinstance(other, Security):
            return self.get_id() == other.get_id()
        else:
            return False

    def __lt__(self, other):
        if isinstance(other, Security):
            return self.get_id() < other.get_id()
        else:
            return False

    def __hash__(self):
        return hash(self.get_id())
